@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Andreas Persson
+ * Copyright (C) 2006-2009 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +18,7 @@
  */
 
 #include <iostream>
+#include <cstring>
 
 #include <gtkmm/filechooserdialog.h>
 #include <gtkmm/messagedialog.h>
@@ -25,6 +26,7 @@
 #include <gtkmm/targetentry.h>
 #include <gtkmm/main.h>
 #include <gtkmm/toggleaction.h>
+#include "wrapLabel.hh"
 
 #include "global.h"
 
@@ -59,6 +61,31 @@ template<class T> inline std::string ToString(T o) {
     return ss.str();
 }
 
+Table::Table(int x, int y) : Gtk::Table(x, y), rowno(0) {  }
+
+void Table::add(BoolEntry& boolentry)
+{
+    attach(boolentry.widget, 0, 2, rowno, rowno + 1,
+           Gtk::FILL, Gtk::SHRINK);
+    rowno++;
+}
+
+void Table::add(BoolEntryPlus6& boolentry)
+{
+    attach(boolentry.widget, 0, 2, rowno, rowno + 1,
+           Gtk::FILL, Gtk::SHRINK);
+    rowno++;
+}
+
+void Table::add(LabelWidget& prop)
+{
+    attach(prop.label, 1, 2, rowno, rowno + 1,
+           Gtk::FILL, Gtk::SHRINK);
+    attach(prop.widget, 2, 3, rowno, rowno + 1,
+           Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK);
+    rowno++;
+}
+
 MainWindow::MainWindow() :
     dimreg_label(_("Changes apply to:")),
     dimreg_all_regions(_("all regions")),
@@ -99,12 +126,12 @@ MainWindow::MainWindow() :
     dimreg_stereo.set_active();
     dimreg_hbox.add(dimreg_stereo);
     dimreg_vbox.add(dimreg_edit);
-    dimreg_vbox.add(dimreg_hbox);
+    dimreg_vbox.pack_start(dimreg_hbox, Gtk::PACK_SHRINK);
     m_HPaned.add2(dimreg_vbox);
 
 
-    m_TreeViewNotebook.append_page(m_ScrolledWindowSamples, "Samples");
-    m_TreeViewNotebook.append_page(m_ScrolledWindow, "Instruments");
+    m_TreeViewNotebook.append_page(m_ScrolledWindowSamples, _("Samples"));
+    m_TreeViewNotebook.append_page(m_ScrolledWindow, _("Instruments"));
 
 
     actionGroup = Gtk::ActionGroup::create();
@@ -176,13 +203,18 @@ MainWindow::MainWindow() :
         sigc::mem_fun(*this, &MainWindow::on_action_add_group)
     );
     actionGroup->add(
-        Gtk::Action::create("AddSample", _("Add _Sample(s)")),
+        Gtk::Action::create("AddSample", _("Add _Sample(s)...")),
         sigc::mem_fun(*this, &MainWindow::on_action_add_sample)
     );
     actionGroup->add(
         Gtk::Action::create("RemoveSample", Gtk::Stock::REMOVE),
         sigc::mem_fun(*this, &MainWindow::on_action_remove_sample)
     );
+    actionGroup->add(
+        Gtk::Action::create("ReplaceAllSamplesInAllGroups",
+                            _("Replace All Samples in All Groups...")),
+        sigc::mem_fun(*this, &MainWindow::on_action_replace_all_samples_in_all_groups)
+    );
 
     uiManager = Gtk::UIManager::create();
     uiManager->insert_action_group(actionGroup);
@@ -223,6 +255,7 @@ MainWindow::MainWindow() :
         "    <menuitem action='SampleProperties'/>"
         "    <menuitem action='AddGroup'/>"
         "    <menuitem action='AddSample'/>"
+	"    <menuitem action='ReplaceAllSamplesInAllGroups' />"
         "    <separator/>"
         "    <menuitem action='RemoveSample'/>"
         "  </popup>"
@@ -235,6 +268,7 @@ MainWindow::MainWindow() :
     m_VBox.pack_start(*menuBar, Gtk::PACK_SHRINK);
     m_VBox.pack_start(m_HPaned);
     m_VBox.pack_start(m_RegionChooser, Gtk::PACK_SHRINK);
+    m_VBox.pack_start(m_RegionChooser.m_VirtKeybPropsBox, Gtk::PACK_SHRINK);
     m_VBox.pack_start(m_DimRegionChooser, Gtk::PACK_SHRINK);
     m_VBox.pack_start(m_StatusBar, Gtk::PACK_SHRINK);
 
@@ -295,6 +329,8 @@ MainWindow::MainWindow() :
         sigc::mem_fun(*this, &MainWindow::file_changed));
     instrumentProps.signal_instrument_changed().connect(
         sigc::mem_fun(*this, &MainWindow::file_changed));
+    propDialog.signal_info_changed().connect(
+        sigc::mem_fun(*this, &MainWindow::file_changed));
 
     dimreg_edit.signal_dimreg_to_be_changed().connect(
         dimreg_to_be_changed_signal.make_slot());
@@ -324,6 +360,11 @@ MainWindow::MainWindow() :
     m_RegionChooser.signal_region_changed_signal().connect(
         region_changed_signal.make_slot());
 
+    note_on_signal.connect(
+        sigc::mem_fun(m_RegionChooser, &RegionChooser::on_note_on_event));
+    note_off_signal.connect(
+        sigc::mem_fun(m_RegionChooser, &RegionChooser::on_note_off_event));
+
     dimreg_all_regions.signal_toggled().connect(
         sigc::mem_fun(*this, &MainWindow::update_dimregs));
     dimreg_all_dimregs.signal_toggled().connect(
@@ -530,7 +571,7 @@ void MainWindow::on_action_file_new()
     gig::File* pFile = new gig::File;
     // already add one new instrument by default
     gig::Instrument* pInstrument = pFile->AddInstrument();
-    pInstrument->pInfo->Name = "Unnamed Instrument";
+    pInstrument->pInfo->Name = _("Unnamed Instrument");
     // update GUI with that new gig::File
     load_gig(pFile, 0 /*no file name yet*/);
 }
@@ -585,22 +626,22 @@ void MainWindow::on_action_file_open()
     Gtk::FileFilter filter;
     filter.add_pattern("*.gig");
     dialog.set_filter(filter);
-    if (current_dir != "") {
-        dialog.set_current_folder(current_dir);
+    if (current_gig_dir != "") {
+        dialog.set_current_folder(current_gig_dir);
     }
     if (dialog.run() == Gtk::RESPONSE_OK) {
         std::string filename = dialog.get_filename();
         printf("filename=%s\n", filename.c_str());
         printf("on_action_file_open self=%x\n", Glib::Thread::self());
         load_file(filename.c_str());
-        current_dir = Glib::path_get_dirname(filename);
+        current_gig_dir = Glib::path_get_dirname(filename);
     }
 }
 
 void MainWindow::load_file(const char* name)
 {
     __clear();
-    load_dialog = new LoadDialog("Loading...", *this);
+    load_dialog = new LoadDialog(_("Loading..."), *this);
     load_dialog->show_all();
     loader = new Loader(strdup(name));
     loader->signal_progress().connect(
@@ -714,12 +755,38 @@ bool MainWindow::file_save_as()
     filter.add_pattern("*.gig");
     dialog.set_filter(filter);
 
-    if (Glib::path_is_absolute(filename)) {
-        dialog.set_filename(filename);
-    } else if (current_dir != "") {
-        dialog.set_current_folder(current_dir);
+    // set initial dir and filename of the Save As dialog
+    // and prepare that initial filename as a copy of the gig
+    {
+        std::string basename = Glib::path_get_basename(filename);
+        std::string dir = Glib::path_get_dirname(filename);
+        basename = std::string(_("copy_of_")) + basename;
+        Glib::ustring copyFileName = Glib::build_filename(dir, basename);
+        if (Glib::path_is_absolute(filename)) {
+            dialog.set_filename(copyFileName);
+        } else {
+            if (current_gig_dir != "") dialog.set_current_folder(current_gig_dir);
+        }
+        dialog.set_current_name(Glib::filename_display_basename(copyFileName));
     }
-    dialog.set_current_name(Glib::filename_display_basename(filename));
+
+    // show warning in the dialog
+    Gtk::HBox descriptionArea;
+    descriptionArea.set_spacing(15);
+    Gtk::Image warningIcon(Gtk::Stock::DIALOG_WARNING, Gtk::IconSize(Gtk::ICON_SIZE_DIALOG));
+    descriptionArea.pack_start(warningIcon, Gtk::PACK_SHRINK);
+    view::WrapLabel description;
+    description.set_markup(
+        _("\n<b>CAUTION:</b> You <b>MUST</b> use the "
+          "<span style=\"italic\">\"Save\"</span> dialog instead of "
+          "<span style=\"italic\">\"Save As...\"</span> if you want to save "
+          "to the same .gig file. Using "
+          "<span style=\"italic\">\"Save As...\"</span> for writing to the "
+          "same .gig file will end up in corrupted sample wave data!\n")
+    );
+    descriptionArea.pack_start(description);
+    dialog.get_vbox()->pack_start(descriptionArea, Gtk::PACK_SHRINK);
+    descriptionArea.show_all();
 
     if (dialog.run() == Gtk::RESPONSE_OK) {
         file_structure_to_be_changed_signal.emit(this->file);
@@ -731,7 +798,7 @@ bool MainWindow::file_save_as()
             printf("filename=%s\n", filename.c_str());
             file->Save(filename);
             this->filename = filename;
-            current_dir = Glib::path_get_dirname(filename);
+            current_gig_dir = Glib::path_get_dirname(filename);
             set_title(Glib::filename_display_basename(filename));
             file_has_name = true;
             file_is_changed = false;
@@ -761,7 +828,7 @@ void MainWindow::__import_queued_samples() {
         info.format = 0;
         SNDFILE* hFile = sf_open((*iter).sample_path.c_str(), SFM_READ, &info);
         try {
-            if (!hFile) throw std::string("could not open file");
+            if (!hFile) throw std::string(_("could not open file"));
             // determine sample's bit depth
             int bitdepth;
             switch (info.format & 0xff) {
@@ -778,7 +845,7 @@ void MainWindow::__import_queued_samples() {
                     break;
                 default:
                     sf_close(hFile); // close sound file
-                    throw std::string("format not supported"); // unsupported subformat (yet?)
+                    throw std::string(_("format not supported")); // unsupported subformat (yet?)
             }
 
             const int bufsize = 10000;
@@ -820,6 +887,8 @@ void MainWindow::__import_queued_samples() {
             }
             // cleanup
             sf_close(hFile);
+            // let the sampler re-cache the sample if needed
+            sample_changed_signal.emit(iter->gig_sample);
             // on success we remove the sample from the import queue,
             // otherwise keep it, maybe it works the next time ?
             std::list<SampleImportItem>::iterator cur = iter;
@@ -850,16 +919,21 @@ void MainWindow::on_action_help_about()
 {
 #ifdef ABOUT_DIALOG
     Gtk::AboutDialog dialog;
+#if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION >= 12) || GTKMM_MAJOR_VERSION > 2
+    dialog.set_program_name("Gigedit");
+#else
+    dialog.set_name("Gigedit");
+#endif
     dialog.set_version(VERSION);
-    dialog.set_copyright("Copyright (C) 2006,2007 Andreas Persson");
-    dialog.set_comments(
+    dialog.set_copyright("Copyright (C) 2006-2009 Andreas Persson");
+    dialog.set_comments(_(
         "Released under the GNU General Public License.\n"
         "\n"
         "Please notice that this is still a very young instrument editor. "
         "So better backup your Gigasampler files before editing them with "
         "this application.\n"
         "\n"
-        "Please report bugs to: http://bugs.linuxsampler.org"
+        "Please report bugs to: http://bugs.linuxsampler.org")
     );
     dialog.set_website("http://www.linuxsampler.org");
     dialog.set_website_label("http://www.linuxsampler.org");
@@ -868,59 +942,108 @@ void MainWindow::on_action_help_about()
 }
 
 PropDialog::PropDialog()
-    : table(2,1)
+    : table(2,1),
+      eName(_("Name")),
+      eCreationDate(_("Creation date")),
+      eComments(_("Comments")),
+      eProduct(_("Product")),
+      eCopyright(_("Copyright")),
+      eArtists(_("Artists")),
+      eGenre(_("Genre")),
+      eKeywords(_("Keywords")),
+      eEngineer(_("Engineer")),
+      eTechnician(_("Technician")),
+      eSoftware(_("Software")),
+      eMedium(_("Medium")),
+      eSource(_("Source")),
+      eSourceForm(_("Source form")),
+      eCommissioned(_("Commissioned")),
+      eSubject(_("Subject")),
+      quitButton(Gtk::Stock::CLOSE),
+      update_model(0)
 {
-    table.set_col_spacings(5);
-    const char* propLabels[] = {
-        "Name:",
-        "CreationDate:",
-        "Comments:", // TODO: multiline
-        "Product:",
-        "Copyright:",
-        "Artists:",
-        "Genre:",
-        "Keywords:",
-        "Engineer:",
-        "Technician:",
-        "Software:", // TODO: readonly
-        "Medium:",
-        "Source:",
-        "SourceForm:",
-        "Commissioned:",
-        "Subject:"
-    };
-    for (int i = 0 ; i < sizeof(propLabels) / sizeof(char*) ; i++) {
-        label[i].set_text(propLabels[i]);
-        label[i].set_alignment(Gtk::ALIGN_LEFT);
-        table.attach(label[i], 0, 1, i, i + 1, Gtk::FILL, Gtk::SHRINK);
-        table.attach(entry[i], 1, 2, i, i + 1, Gtk::FILL | Gtk::EXPAND,
-                     Gtk::SHRINK);
-    }
+    set_title(_("File Properties"));
+    eName.set_width_chars(50);
 
-    add(table);
-    // add_button(Gtk::Stock::CANCEL, 0);
-    // add_button(Gtk::Stock::OK, 1);
+    connect(eName, &DLS::Info::Name);
+    connect(eCreationDate, &DLS::Info::CreationDate);
+    connect(eComments, &DLS::Info::Comments);
+    connect(eProduct, &DLS::Info::Product);
+    connect(eCopyright, &DLS::Info::Copyright);
+    connect(eArtists, &DLS::Info::Artists);
+    connect(eGenre, &DLS::Info::Genre);
+    connect(eKeywords, &DLS::Info::Keywords);
+    connect(eEngineer, &DLS::Info::Engineer);
+    connect(eTechnician, &DLS::Info::Technician);
+    connect(eSoftware, &DLS::Info::Software);
+    connect(eMedium, &DLS::Info::Medium);
+    connect(eSource, &DLS::Info::Source);
+    connect(eSourceForm, &DLS::Info::SourceForm);
+    connect(eCommissioned, &DLS::Info::Commissioned);
+    connect(eSubject, &DLS::Info::Subject);
+
+    table.add(eName);
+    table.add(eCreationDate);
+    table.add(eComments);
+    table.add(eProduct);
+    table.add(eCopyright);
+    table.add(eArtists);
+    table.add(eGenre);
+    table.add(eKeywords);
+    table.add(eEngineer);
+    table.add(eTechnician);
+    table.add(eSoftware);
+    table.add(eMedium);
+    table.add(eSource);
+    table.add(eSourceForm);
+    table.add(eCommissioned);
+    table.add(eSubject);
+
+    table.set_col_spacings(5);
+    add(vbox);
+    table.set_border_width(5);
+    vbox.add(table);
+    vbox.pack_start(buttonBox, Gtk::PACK_SHRINK);
+    buttonBox.set_layout(Gtk::BUTTONBOX_END);
+    buttonBox.set_border_width(5);
+    buttonBox.show();
+    buttonBox.pack_start(quitButton);
+    quitButton.set_flags(Gtk::CAN_DEFAULT);
+    quitButton.grab_focus();
+    quitButton.signal_clicked().connect(
+        sigc::mem_fun(*this, &PropDialog::hide));
+
+    quitButton.show();
+    vbox.show();
     show_all_children();
 }
 
 void PropDialog::set_info(DLS::Info* info)
 {
-    entry[0].set_text(info->Name);
-    entry[1].set_text(info->CreationDate);
-    entry[2].set_text(Glib::convert(info->Comments, "UTF-8", "ISO-8859-1"));
-    entry[3].set_text(info->Product);
-    entry[4].set_text(info->Copyright);
-    entry[5].set_text(info->Artists);
-    entry[6].set_text(info->Genre);
-    entry[7].set_text(info->Keywords);
-    entry[8].set_text(info->Engineer);
-    entry[9].set_text(info->Technician);
-    entry[10].set_text(info->Software);
-    entry[11].set_text(info->Medium);
-    entry[12].set_text(info->Source);
-    entry[13].set_text(info->SourceForm);
-    entry[14].set_text(info->Commissioned);
-    entry[15].set_text(info->Subject);
+    this->info = info;
+    update_model++;
+    eName.set_value(info->Name);
+    eCreationDate.set_value(info->CreationDate);
+    eComments.set_value(info->Comments);
+    eProduct.set_value(info->Product);
+    eCopyright.set_value(info->Copyright);
+    eArtists.set_value(info->Artists);
+    eGenre.set_value(info->Genre);
+    eKeywords.set_value(info->Keywords);
+    eEngineer.set_value(info->Engineer);
+    eTechnician.set_value(info->Technician);
+    eSoftware.set_value(info->Software);
+    eMedium.set_value(info->Medium);
+    eSource.set_value(info->Source);
+    eSourceForm.set_value(info->SourceForm);
+    eCommissioned.set_value(info->Commissioned);
+    eSubject.set_value(info->Subject);
+    update_model--;
+}
+
+sigc::signal<void>& PropDialog::signal_info_changed()
+{
+    return info_changed;
 }
 
 void InstrumentProps::set_IsDrum(bool value)
@@ -954,47 +1077,33 @@ void InstrumentProps::set_DimensionKeyRange_high(uint8_t value)
     }
 }
 
-void InstrumentProps::add_prop(BoolEntry& boolentry)
-{
-    table.attach(boolentry.widget, 0, 2, rowno, rowno + 1,
-                 Gtk::FILL, Gtk::SHRINK);
-    rowno++;
-}
-
-void InstrumentProps::add_prop(BoolEntryPlus6& boolentry)
-{
-    table.attach(boolentry.widget, 0, 2, rowno, rowno + 1,
-                 Gtk::FILL, Gtk::SHRINK);
-    rowno++;
-}
-
-void InstrumentProps::add_prop(LabelWidget& prop)
-{
-    table.attach(prop.label, 0, 1, rowno, rowno + 1,
-                 Gtk::FILL, Gtk::SHRINK);
-    table.attach(prop.widget, 1, 2, rowno, rowno + 1,
-                 Gtk::FILL | Gtk::EXPAND, Gtk::SHRINK);
-    rowno++;
-}
-
 InstrumentProps::InstrumentProps()
     : table(2,1),
       quitButton(Gtk::Stock::CLOSE),
-      eName("Name"),
-      eIsDrum("Is drum"),
-      eMIDIBank("MIDI bank", 0, 16383),
-      eMIDIProgram("MIDI program"),
-      eAttenuation("Attenuation", 0, 96, 0, 1),
-      eGainPlus6("Gain +6dB", eAttenuation, -6),
-      eEffectSend("Effect send", 0, 65535),
-      eFineTune("Fine tune", -8400, 8400),
-      ePitchbendRange("Pitchbend range", 0, 12),
-      ePianoReleaseMode("Piano release mode"),
-      eDimensionKeyRangeLow("Dimension key range low"),
-      eDimensionKeyRangeHigh("Dimension key range high"),
+      eName(_("Name")),
+      eIsDrum(_("Is drum")),
+      eMIDIBank(_("MIDI bank"), 0, 16383),
+      eMIDIProgram(_("MIDI program")),
+      eAttenuation(_("Attenuation"), 0, 96, 0, 1),
+      eGainPlus6(_("Gain +6dB"), eAttenuation, -6),
+      eEffectSend(_("Effect send"), 0, 65535),
+      eFineTune(_("Fine tune"), -8400, 8400),
+      ePitchbendRange(_("Pitchbend range"), 0, 12),
+      ePianoReleaseMode(_("Piano release mode")),
+      eDimensionKeyRangeLow(_("Keyswitching range low")),
+      eDimensionKeyRangeHigh(_("Keyswitching range high")),
       update_model(0)
 {
-    set_title("Instrument properties");
+    set_title(_("Instrument Properties"));
+
+    eDimensionKeyRangeLow.set_tip(
+        _("start of the keyboard area which should switch the "
+          "\"keyswitching\" dimension")
+    );
+    eDimensionKeyRangeHigh.set_tip(
+        _("end of the keyboard area which should switch the "
+          "\"keyswitching\" dimension")
+    );
 
     connect(eIsDrum, &InstrumentProps::set_IsDrum);
     connect(eMIDIBank, &InstrumentProps::set_MIDIBank);
@@ -1010,21 +1119,20 @@ InstrumentProps::InstrumentProps()
     connect(eDimensionKeyRangeHigh,
             &InstrumentProps::set_DimensionKeyRange_high);
 
-    rowno = 0;
     table.set_col_spacings(5);
 
-    add_prop(eName);
-    add_prop(eIsDrum);
-    add_prop(eMIDIBank);
-    add_prop(eMIDIProgram);
-    add_prop(eAttenuation);
-    add_prop(eGainPlus6);
-    add_prop(eEffectSend);
-    add_prop(eFineTune);
-    add_prop(ePitchbendRange);
-    add_prop(ePianoReleaseMode);
-    add_prop(eDimensionKeyRangeLow);
-    add_prop(eDimensionKeyRangeHigh);
+    table.add(eName);
+    table.add(eIsDrum);
+    table.add(eMIDIBank);
+    table.add(eMIDIProgram);
+    table.add(eAttenuation);
+    table.add(eGainPlus6);
+    table.add(eEffectSend);
+    table.add(eFineTune);
+    table.add(ePitchbendRange);
+    table.add(ePianoReleaseMode);
+    table.add(eDimensionKeyRangeLow);
+    table.add(eDimensionKeyRangeHigh);
 
     add(vbox);
     table.set_border_width(5);
@@ -1051,7 +1159,7 @@ void InstrumentProps::set_instrument(gig::Instrument* instrument)
     this->instrument = instrument;
 
     update_model++;
-    eName.set_ptr(&instrument->pInfo->Name);
+    eName.set_value(instrument->pInfo->Name);
     eIsDrum.set_value(instrument->IsDrum);
     eMIDIBank.set_value(instrument->MIDIBank);
     eMIDIProgram.set_value(instrument->MIDIProgram);
@@ -1211,7 +1319,7 @@ void MainWindow::on_action_add_instrument() {
     gig::Instrument* instrument = file->AddInstrument();
     __instrument_indexer++;
     instrument->pInfo->Name =
-        "Unnamed Instrument " + ToString(__instrument_indexer);
+        _("Unnamed Instrument ") + ToString(__instrument_indexer);
     // update instrument tree view
     Gtk::TreeModel::iterator iterInstr = m_refTreeModel->append();
     Gtk::TreeModel::Row rowInstr = *iterInstr;
@@ -1254,7 +1362,7 @@ void MainWindow::on_action_remove_instrument() {
 void MainWindow::on_action_sample_properties() {
     //TODO: show a dialog where the selected sample's properties can be edited
     Gtk::MessageDialog msg(
-        *this, "Sorry, yet to be implemented!", false, Gtk::MESSAGE_INFO
+        *this, _("Sorry, yet to be implemented!"), false, Gtk::MESSAGE_INFO
     );
     msg.run();
 }
@@ -1263,7 +1371,7 @@ void MainWindow::on_action_add_group() {
     static int __sample_indexer = 0;
     if (!file) return;
     gig::Group* group = file->AddGroup();
-    group->Name = "Unnamed Group";
+    group->Name = _("Unnamed Group");
     if (__sample_indexer) group->Name += " " + ToString(__sample_indexer);
     __sample_indexer++;
     // update sample tree view
@@ -1307,13 +1415,17 @@ void MainWindow::on_action_add_sample() {
     };
     for (int i = 0; supportedFileTypes[i]; i++)
         soundfilter.add_pattern(supportedFileTypes[i]);
-    soundfilter.set_name("Sound Files");
+    soundfilter.set_name(_("Sound Files"));
     Gtk::FileFilter allpassfilter; // matches every file
     allpassfilter.add_pattern("*.*");
-    allpassfilter.set_name("All Files");
+    allpassfilter.set_name(_("All Files"));
     dialog.add_filter(soundfilter);
     dialog.add_filter(allpassfilter);
+    if (current_sample_dir != "") {
+        dialog.set_current_folder(current_sample_dir);
+    }
     if (dialog.run() == Gtk::RESPONSE_OK) {
+        current_sample_dir = dialog.get_current_folder();
         Glib::ustring error_files;
         Glib::SListHandle<Glib::ustring> filenames = dialog.get_filenames();
         for (Glib::SListHandle<Glib::ustring>::iterator iter = filenames.begin();
@@ -1324,7 +1436,7 @@ void MainWindow::on_action_add_sample() {
             info.format = 0;
             SNDFILE* hFile = sf_open((*iter).c_str(), SFM_READ, &info);
             try {
-                if (!hFile) throw std::string("could not open file");
+                if (!hFile) throw std::string(_("could not open file"));
                 int bitdepth;
                 switch (info.format & 0xff) {
                     case SF_FORMAT_PCM_S8:
@@ -1340,7 +1452,7 @@ void MainWindow::on_action_add_sample() {
                         break;
                     default:
                         sf_close(hFile); // close sound file
-                        throw std::string("format not supported"); // unsupported subformat (yet?)
+                        throw std::string(_("format not supported")); // unsupported subformat (yet?)
                 }
                 // add a new sample to the .gig file
                 gig::Sample* sample = file->AddSample();
@@ -1426,6 +1538,100 @@ void MainWindow::on_action_add_sample() {
     }
 }
 
+void MainWindow::on_action_replace_all_samples_in_all_groups()
+{
+    if (!file) return;
+    Gtk::FileChooserDialog dialog(*this, _("Select Folder"),
+                                  Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
+    view::WrapLabel description(
+        _("This is a very specific function. It tries to replace all samples "
+          "in the current gig file by samples located in the chosen "
+          "directory.\n\n"
+          "It works like this: For each sample in the gig file, it tries to "
+          "find a sample file in the selected directory with the same name as "
+          "the sample in the gig file. Optionally, you can add a filename "
+          "extension below, which will be added to the filename expected to be "
+          "found. That is, assume you have a gig file with a sample called "
+          "'Snare', if you enter '.wav' below (like it's done by default), it "
+          "expects to find a sample file called 'Snare.wav' and will replace "
+          "the sample in the gig file accordingly. If you don't need an "
+          "extension, blank the field below. Any gig sample where no "
+          "appropriate sample file could be found will be reported and left "
+          "untouched.\n")
+    );
+    Gtk::HBox entryArea;
+    Gtk::Label entryLabel( _("Add filename extension: "), Gtk::ALIGN_RIGHT);
+    Gtk::Entry postfixEntryBox;
+    postfixEntryBox.set_text(".wav");
+    entryArea.pack_start(entryLabel);
+    entryArea.pack_start(postfixEntryBox);
+    dialog.get_vbox()->pack_start(description, Gtk::PACK_SHRINK);
+    dialog.get_vbox()->pack_start(entryArea, Gtk::PACK_SHRINK);
+    description.show();
+    entryArea.show_all();
+    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+    dialog.add_button(_("Select"), Gtk::RESPONSE_OK);
+    dialog.set_select_multiple(false);
+    if (current_sample_dir != "") {
+        dialog.set_current_folder(current_sample_dir);
+    }
+    if (dialog.run() == Gtk::RESPONSE_OK)
+    {
+        current_sample_dir = dialog.get_current_folder();
+        Glib::ustring error_files;
+        Glib::ustring folder = dialog.get_filename();
+        for (gig::Sample* sample = file->GetFirstSample();
+             sample; sample = file->GetNextSample())
+        {
+            std::string filename =
+                folder + G_DIR_SEPARATOR_S + sample->pInfo->Name +
+                postfixEntryBox.get_text().raw();
+            SF_INFO info;
+            info.format = 0;
+            SNDFILE* hFile = sf_open(filename.c_str(), SFM_READ, &info);
+            try
+            {
+                if (!hFile) throw std::string(_("could not open file"));
+                int bitdepth;
+                switch (info.format & 0xff) {
+                    case SF_FORMAT_PCM_S8:
+                    case SF_FORMAT_PCM_16:
+                    case SF_FORMAT_PCM_U8:
+                        bitdepth = 16;
+                        break;
+                    case SF_FORMAT_PCM_24:
+                    case SF_FORMAT_PCM_32:
+                    case SF_FORMAT_FLOAT:
+                    case SF_FORMAT_DOUBLE:
+                        bitdepth = 24;
+                        break;
+                    default:
+                        sf_close(hFile);
+                        throw std::string(_("format not supported"));
+                }
+                SampleImportItem sched_item;
+                sched_item.gig_sample  = sample;
+                sched_item.sample_path = filename;
+                m_SampleImportQueue.push_back(sched_item);
+                sf_close(hFile);
+                file_changed();
+            }
+            catch (std::string what)
+            {
+                if (error_files.size()) error_files += "\n";
+                    error_files += filename += " (" + what + ")";
+            }
+        }
+        // show error message box when some file(s) could not be opened / added
+        if (error_files.size()) {
+            Glib::ustring txt =
+                _("Could not replace the following sample(s):\n") + error_files;
+            Gtk::MessageDialog msg(*this, txt, false, Gtk::MESSAGE_ERROR);
+            msg.run();
+        }
+    }
+}
+
 void MainWindow::on_action_remove_sample() {
     if (!file) return;
     Glib::RefPtr<Gtk::TreeSelection> sel = m_TreeViewSamples.get_selection();
@@ -1674,6 +1880,10 @@ sigc::signal<void, gig::Region*>& MainWindow::signal_region_changed() {
     return region_changed_signal;
 }
 
+sigc::signal<void, gig::Sample*>& MainWindow::signal_sample_changed() {
+    return sample_changed_signal;
+}
+
 sigc::signal<void, gig::Sample*/*old*/, gig::Sample*/*new*/>& MainWindow::signal_sample_ref_changed() {
     return sample_ref_changed_signal;
 }
@@ -1685,3 +1895,19 @@ sigc::signal<void, gig::DimensionRegion*>& MainWindow::signal_dimreg_to_be_chang
 sigc::signal<void, gig::DimensionRegion*>& MainWindow::signal_dimreg_changed() {
     return dimreg_changed_signal;
 }
+
+sigc::signal<void, int/*key*/, int/*velocity*/>& MainWindow::signal_note_on() {
+    return note_on_signal;
+}
+
+sigc::signal<void, int/*key*/, int/*velocity*/>& MainWindow::signal_note_off() {
+    return note_off_signal;
+}
+
+sigc::signal<void, int/*key*/, int/*velocity*/>& MainWindow::signal_keyboard_key_hit() {
+    return m_RegionChooser.signal_keyboard_key_hit();
+}
+
+sigc::signal<void, int/*key*/, int/*velocity*/>& MainWindow::signal_keyboard_key_released() {
+    return m_RegionChooser.signal_keyboard_key_released();
+}
