@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007 Andreas Persson
+ * Copyright (C) 2007, 2008 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,14 +29,21 @@
 class LinuxSamplerPlugin : public LinuxSampler::InstrumentEditor {
     public:
         LinuxSamplerPlugin();
-        virtual int Main(void* pInstrument, String sTypeName, String sTypeVersion);
+        virtual int Main(void* pInstrument, String sTypeName, String sTypeVersion); // old LS plugin API entry point
+        virtual int Main(void* pInstrument, String sTypeName, String sTypeVersion, void* pUserData); // new LS plugin API entry point
         virtual bool IsTypeSupported(String sTypeName, String sTypeVersion);
         virtual String Name();
         virtual String Version();
         virtual String Description();
+        virtual ~LinuxSamplerPlugin();
 
     private:
+        void* pApp;
+
         void __onSamplesToBeRemoved(std::list<gig::Sample*> lSamples);
+        void __onVirtualKeyboardKeyHit(int Key, int Velocity);
+        void __onVirtualKeyboardKeyReleased(int Key, int Velocity);
+        bool __onPollPeriod();
 };
 
 #endif // GIGEDIT_LINUXSAMPLER_PLUGIN_H
